
var pPort;
var fSendResponse;
var hostChecked = false;
var hostVers = null;

var promisesCallbacks = new Array();

var already_get_responses_from_native = false;

browser.runtime.onMessage.addListener((req, sender, sendReply) => {
  
    var request = req["contentMessage"];
    console.log("recv message from page:" + JSON.stringify(request));

    if (request["command"] === "disc") {
        disconnectHost();
        return Promise.resolve({contentMessageReply: {"status": "ok"}});
    }    
    
    if (request["command"] === "checkExt") {
        console.log("checkExt");
        return Promise.resolve({contentMessageReply: {"status": "ok", "vers":browser.runtime.getManifest().version}});
    }
    
    if (request["command"] === "sendToLs") {
        return sendHttpRequestToLS(request["arg"]);
    }    

    if (request["command"] === "checkHost") {
        console.log("checkHost");
        return checkHost();
    } 
    
    
    try {
        if (pPort == null) {
            connectToHost();
        }
    } catch (ex) {
        return Promise.reject({contentMessageReply: {ex}});
    }
    
    
    var p1 = new Promise(function(resolve, reject) {
        try {
            // TODO: add timeout to reject commands
            var comm = request["command"];
            promisesCallbacks[comm] = new Array();
            promisesCallbacks[comm]["resolve"] = resolve;
            promisesCallbacks[comm]["reject"] = reject;
            
            sendNativeMessage(request);
        } catch (ex) {
            console.log(ex);
            reject({contentMessageReply: ex});
        }
    });    
    return p1;
    
});

function checkHost() {
        
    if (hostChecked || pPort != null) {
        hostChecked = true;
        console.log("checkHost good: port != null");
        return Promise.resolve({contentMessageReply: {"command":"checkHost", "status": "ok", "vers":hostVers}});
    }
    
    var p1 = new Promise(function(resolve, reject) {
        
        var sending = browser.runtime.sendNativeMessage("kz.akkamal.clientloader", { command: "getVersion" });
        sending.then(
            function (response) {
                try {
                    if (response) {
                        console.log("Received " + JSON.stringify(response));
                        hostVers = response["vers"];
                        resolve({contentMessageReply:{"command":"checkHost", "status":"ok", "vers":response["vers"]}});
                    } else {
                        console.log("response == null:" + browser.runtime.lastError.message);
                        resolve({contentMessageReply:{"command":"checkHost", "status":"fail"}});
                    }
                } catch (ex) {
                    console.log("ERROR:" + ex);
                    resolve({contentMessageReply:{"command":"checkHost", "status":"fail"}});
                }
            }, 
            function(errMess) {
                resolve({contentMessageReply:{"command":"checkHost", "status":"fail"}});
            }
        );           
    });    
    return p1;    
}



function connectToHost() {
    var hostName = "kz.akkamal.clientloader";
    console.log("Connecting to native messaging host: " + hostName)
    pPort = browser.runtime.connectNative(hostName);
    pPort.onMessage.addListener(onNativeMessage);
    pPort.onDisconnect.addListener(onDisconnected);

    // initiate pinging
    ping();
}

function disconnectHost() {
    if (pPort != null) {
        pPort.disconnect();
        pPort = null;
        console.log("port disconnected");
    }
}

function ping() {
    if (pPort != null) {
        var comm = {
            command: "ping",
        };

        sendNativeMessage(comm);
        setTimeout(ping, 5000);
    }
}

function sendHttpRequestToLS(reqObj, callback) {

    var p1 = new Promise(function(resolve, reject) {
        
        console.log("sendHttpRequestToLs");
        var httpReq = new XMLHttpRequest();
        httpReq.open("POST", reqObj.url);
        httpReq.setRequestHeader("Content-Type", "application/json; charset=utf-8");

        httpReq.onreadystatechange = function () {
            console.log("httpReqReady: " + httpReq.status + ":" + httpReq.responseText);
            var DONE = 4;
            var OK = 200;
            if (httpReq.readyState === DONE) {
                if (httpReq.status === OK) {
                    console.log(httpReq.responseText);
                    resolve({contentMessageReply:{res: "success", obj: JSON.parse(httpReq.responseText)}});
                } else {
                    console.log('Error: ' + httpReq.status);
                    resolve({contentMessageReply:{res: "error", errMess: 'Error: ' + httpReq.status}});
                }
            }
        }
        httpReq.send(reqObj.data);
    });
    return p1;
}

function sendNativeMessage(message) {
    pPort.postMessage(message);
    console.log("Sent message to host: " + JSON.stringify(message));
}

function onNativeMessage(message) {
    already_get_responses_from_native = true;
    
    console.log("Received message from host: " + JSON.stringify(message));
    var comm = message["command"];
    
    promisesCallbacks[comm]["resolve"]({contentMessageReply: message});
    
}

function onDisconnected() {
    if (browser.runtime.lastError) {
        alert("Failed to connect: " + browser.runtime.lastError.message);
        console.log("Failed to connect: " + browser.runtime.lastError.message);
    }
    
    var portError = "";
    try {
        portError =  pPort.error;
    } catch (ex) {
        console.log("ex:" + ex);
    }
    
    if (already_get_responses_from_native === false) {
        try {
            promisesCallbacks["start"]["resolve"]({errorMessageReply:"Host application (.exe or .bin file) not installed. Go to https://e-security.kz/web_loader/ download and install it. ERROR: " + portError});
        } catch (ex) {
            console.log("reject err:" + ex);
        }
    }
    pPort = null;
}

console.log("ext started");